/*
 * See the README file for copyright information and how to reach the author.
 *
 */

#include "common.h"

static const char* boolValues[] =
{
    "0",
    "1"
};

static const char* videoStandardValues[] =
{
    "NTSC-M",
    "SECAM-L",
    "PAL-BG",
    "PAL-I",
    "PAL-DK"
};

static const char* videoDeviceValues[] =
{
    "/dev/video0",
    "/dev/video1",
    "/dev/video2",
    "/dev/video3",
    "/dev/video4",
    "/dev/video5",
    "/dev/video6",
    "/dev/video7"
};

static const char* videoResolutionValues[] =
{
    "352x288",
    "352x576",
    "480x576",
    "544x576",
    "704x576",
    "720x576"
};

static const char* audioBitrateValues[] =
{
    "96kb/s",
    "112kb/s",
    "128kb/s",
    "160kb/s",
    "192kb/s",
    "224kb/s",
    "256kb/s",
    "320kb/s",
    "384kb/s"
};

static const char* videoBitrateValues[] =
{
    "1.0Mb/s",
    "1.1Mb/s",
    "1.2Mb/s",
    "1.3Mb/s",
    "1.4Mb/s",
    "1.5Mb/s",
    "1.6Mb/s",
    "1.7Mb/s",
    "1.8Mb/s",
    "1.9Mb/s",
    "2.0Mb/s",
    "2.1Mb/s",
    "2.2Mb/s",
    "2.3Mb/s",
    "2.4Mb/s",
    "2.5Mb/s",
    "2.6Mb/s",
    "2.7Mb/s",
    "2.8Mb/s",
    "2.9Mb/s",
    "3.0Mb/s",
    "3.1Mb/s",
    "3.2Mb/s",
    "3.3Mb/s",
    "3.4Mb/s",
    "3.5Mb/s",
    "3.6Mb/s",
    "3.7Mb/s",
    "3.8Mb/s",
    "3.9Mb/s",
    "4.0Mb/s",
    "4.2Mb/s",
    "4.4Mb/s",
    "4.6Mb/s",
    "4.8Mb/s",
    "5.0Mb/s",
    "5.2Mb/s",
    "5.4Mb/s",
    "5.6Mb/s",
    "5.8Mb/s",
    "6.0Mb/s",
    "6.2Mb/s",
    "6.4Mb/s",
    "6.6Mb/s",
    "6.8Mb/s",
    "7.0Mb/s",
    "7.5Mb/s",
    "8.0Mb/s",
    "8.5Mb/s",
    "9.0Mb/s",
    "9.5Mb/s",
    "10.0Mb/s",
    "10.5Mb/s",
    "11.0Mb/s",
    "12.0Mb/s",
    "13.0Mb/s",
    "14.0Mb/s",
    "15.0Mb/s",
    "16.0Mb/s",
    "17.0Mb/s",
    "18.0Mb/s",
    "19.0Mb/s",
    "20.0Mb/s"
};

static const char* sliderValues[] =
{
    "[                             ]",
    "[|                            ]",
    "[||                           ]",
    "[|||                          ]",
    "[||||                         ]",
    "[|||||                        ]",
    "[||||||                       ]",
    "[|||||||                      ]",
    "[||||||||                     ]",
    "[|||||||||                    ]",
    "[||||||||||                   ]",
    "[|||||||||||                  ]",
    "[||||||||||||                 ]",
    "[|||||||||||||                ]",
    "[||||||||||||||               ]",
    "[|||||||||||||||              ]",
    "[||||||||||||||||             ]",
    "[|||||||||||||||||            ]",
    "[||||||||||||||||||           ]",
    "[|||||||||||||||||||          ]",
    "[||||||||||||||||||||         ]",
    "[|||||||||||||||||||||        ]",
    "[||||||||||||||||||||||       ]",
    "[|||||||||||||||||||||||      ]",
    "[||||||||||||||||||||||||     ]",
    "[|||||||||||||||||||||||||    ]",
    "[||||||||||||||||||||||||||   ]",
    "[|||||||||||||||||||||||||||  ]",
    "[|||||||||||||||||||||||||||| ]",
    "[|||||||||||||||||||||||||||||]",
};

static const char* bigSliderValues[] =
{
    "[                                                                                ]",
    "[|                                                                               ]",
    "[||                                                                              ]",
    "[|||                                                                             ]",
    "[||||                                                                            ]",
    "[|||||                                                                           ]",
    "[||||||                                                                          ]",
    "[|||||||                                                                         ]",
    "[||||||||                                                                        ]",
    "[|||||||||                                                                       ]",
    "[||||||||||                                                                      ]",
    "[|||||||||||                                                                     ]",
    "[||||||||||||                                                                    ]",
    "[|||||||||||||                                                                   ]",
    "[||||||||||||||                                                                  ]",
    "[|||||||||||||||                                                                 ]",
    "[||||||||||||||||                                                                ]",
    "[|||||||||||||||||                                                               ]",
    "[||||||||||||||||||                                                              ]",
    "[|||||||||||||||||||                                                             ]",
    "[||||||||||||||||||||                                                            ]",
    "[|||||||||||||||||||||                                                           ]",
    "[||||||||||||||||||||||                                                          ]",
    "[|||||||||||||||||||||||                                                         ]",
    "[||||||||||||||||||||||||                                                        ]",
    "[|||||||||||||||||||||||||                                                       ]",
    "[||||||||||||||||||||||||||                                                      ]",
    "[|||||||||||||||||||||||||||                                                     ]",
    "[||||||||||||||||||||||||||||                                                    ]",
    "[|||||||||||||||||||||||||||||                                                   ]",
    "[||||||||||||||||||||||||||||||                                                  ]",
    "[|||||||||||||||||||||||||||||||                                                 ]",
    "[||||||||||||||||||||||||||||||||                                                ]",
    "[|||||||||||||||||||||||||||||||||                                               ]",
    "[||||||||||||||||||||||||||||||||||                                              ]",
    "[|||||||||||||||||||||||||||||||||||                                             ]",
    "[||||||||||||||||||||||||||||||||||||                                            ]",
    "[|||||||||||||||||||||||||||||||||||||                                           ]",
    "[||||||||||||||||||||||||||||||||||||||                                          ]",
    "[|||||||||||||||||||||||||||||||||||||||                                         ]",
    "[||||||||||||||||||||||||||||||||||||||||                                        ]",
    "[|||||||||||||||||||||||||||||||||||||||||                                       ]",
    "[||||||||||||||||||||||||||||||||||||||||||                                      ]",
    "[|||||||||||||||||||||||||||||||||||||||||||                                     ]",
    "[||||||||||||||||||||||||||||||||||||||||||||                                    ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||                                   ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||                                  ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||                                 ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||                                ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||                               ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||                              ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||                             ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||                            ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||                           ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||                          ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||                         ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||                        ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||                       ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||                      ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||                     ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||                    ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||                   ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||                  ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||                 ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||                ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||               ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||              ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||             ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||            ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||           ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||          ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||         ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||        ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||       ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||      ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||     ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||    ]",
    "[|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||   ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||  ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||| ]",
    "[||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||]",
};

static const char* offsetValues[] =
{
    "[|.............|..............]",
    "[.|............|..............]",
    "[..|...........|..............]",
    "[...|..........|..............]",
    "[....|.........|..............]",
    "[.....|........|..............]",
    "[......|.......|..............]",
    "[.......|......|..............]",
    "[........|.....|..............]",
    "[.........|....|..............]",
    "[..........|...|..............]",
    "[...........|..|..............]",
    "[............|.|..............]",
    "[.............||..............]",
    "[..............|..............]",
    "[..............||.............]",
    "[..............|.|............]",
    "[..............|..|...........]",
    "[..............|...|..........]",
    "[..............|....|.........]",
    "[..............|.....|........]",
    "[..............|......|.......]",
    "[..............|.......|......]",
    "[..............|........|.....]",
    "[..............|.........|....]",
    "[..............|..........|...]",
    "[..............|...........|..]",
    "[..............|............|.]",
    "[..............|.............|]"
};

static const char* percentValues[] =
{
    "0%",
    "2%",
    "4%",
    "6%",
    "8%",
    "10%",
    "12%",
    "14%",
    "16%",
    "18%",
    "20%",
    "22%",
    "24%",
    "26%",
    "28%",
    "30%",
    "32%",
    "34%",
    "36%",
    "38%",
    "40%",
    "42%",
    "44%",
    "46%",
    "48%",
    "50%",
    "52%",
    "54%",
    "56%",
    "58%",
    "60%",
    "62%",
    "64%",
    "66%",
    "68%",
    "70%",
    "72%",
    "74%",
    "76%",
    "78%",
    "80%",
    "82%",
    "84%",
    "86%",
    "88%",
    "90%",
    "92%",
    "94%",
    "96%",
    "98%",
    "100%"
};


int cRMSetupTools::NumIndices(eItemType _item)
{
    switch(_item) {
        case eBoolItem:
            return ENTRIES(boolValues, char*);
        case ePercentItem:
            return ENTRIES(percentValues, char*);
        case eSliderItem:
            return ENTRIES(sliderValues, char*);
        case eBigSliderItem:
            return ENTRIES(bigSliderValues, char*);
        case eOffsetItem:
            return ENTRIES(offsetValues, char*);
        case eAudioBitrateItem:
            return ENTRIES(audioBitrateValues, char*);
        case eVideoBitrateItem:
            return ENTRIES(videoBitrateValues, char*);
        case eVideoStandardItem:
            return ENTRIES(videoStandardValues, char*);
        case eVideoDeviceItem:
            return ENTRIES(videoDeviceValues, char*);
        case eVideoResolutionItem:
        case eVideoResolutionHorizontalItem:
        case eVideoResolutionVerticalItem:
            return ENTRIES(videoResolutionValues, char*);
    }
    return -1;
}

const char* const* cRMSetupTools::Values(eItemType _item)
{
    switch(_item) {
        case eBoolItem:
            return boolValues;
        case ePercentItem:
            return percentValues;
        case eSliderItem:
            return sliderValues;
        case eBigSliderItem:
            return bigSliderValues;
        case eOffsetItem:
            return offsetValues;
        case eAudioBitrateItem:
            return audioBitrateValues;
        case eVideoBitrateItem:
            return videoBitrateValues;
        case eVideoStandardItem:
            return videoStandardValues;
        case eVideoDeviceItem:
            return videoDeviceValues;
        case eVideoResolutionItem:
            return videoResolutionValues;
        case eVideoResolutionHorizontalItem:
        case eVideoResolutionVerticalItem:
            return NULL;
    }
    return NULL;
}

int cRMSetupTools::ClipIndex(eItemType _item, int _index)
{
    int numIndices=NumIndices(_item);
    if(numIndices<=0) {
        return -1;
    }
    if(_index<0) {
        return 0;
    }
    if(_index>numIndices) {
        return numIndices-1;
    }
    return _index;
}

char* cRMSetupTools::Index2StringValue(eItemType _item, int _index)
{
    if(_index<0 || _index>=NumIndices(_item)) {
        return NULL;
    }
    switch(_item) {
        case eVideoResolutionHorizontalItem: {
            char* value=Index2StringValue(eVideoResolutionItem, _index);
            char* pos=strchr(value, 'x');
            pos[0]=0;
            return value;
        }
        case eVideoResolutionVerticalItem: {
            char* value=Index2StringValue(eVideoResolutionItem, _index);
            char* pos=strchr(value, 'x')+1;
            memmove(value, pos, strlen(pos)+1);
            return value;
        }
        default: {
            const char* const* values=Values(_item);
            if(!values) {
                return NULL;
            }
            return strdup(values[_index]);
        }
    }
    return NULL;
}

int cRMSetupTools::Index2Value(eItemType _item, int _index)
{
    int numIndices=NumIndices(_item);
    if(_index<0 || _index>=numIndices) {
        return -1;
    }
    switch(_item) {
        case ePercentItem:
        case eSliderItem:
        case eBigSliderItem:
            return _index*100/(numIndices-1);
        case eOffsetItem:
            return _index*200/(numIndices-1)-100;
        case eAudioBitrateItem:
            return AudioBitrateValue(_index);
        case eVideoBitrateItem:
            return VideoBitrateValue(_index);
        case eVideoResolutionHorizontalItem:
        case eVideoResolutionVerticalItem: {
            char* value=Index2StringValue(_item, _index);
            if(!value) {
                return -1;
            }
            int retVal=atoi(value);
            free(value);
            return retVal;
        }
        default:
            return _index;
    }

    return -1;
}

int cRMSetupTools::Index2Value(eItemType _item, int _index,
        int _minValue, int _maxValue)
{
    int value=Index2Value(_item, _index);
    if(_maxValue==-1 && _minValue==-1) {
        return value;
    }
    switch(_item) {
        case ePercentItem:
        case eSliderItem:
        case eBigSliderItem:
            return (int)(1.0*value*(_maxValue-_minValue)/100+_minValue+0.5);
        case eOffsetItem:
            return (int)(1.0*(value+100)*(_maxValue-_minValue)/200+_minValue+0.5);
        default:
            return value;
    }
    return -1;
}

int cRMSetupTools::Value2Index(eItemType _item, const char* _value)
{
    if(!_value) {
        return -1;
    }
    int numIndices=NumIndices(_item);
    const char* const* values=Values(_item);
    int valueLength=strlen(_value);
    for(int i=0; i<numIndices; i++) {
        // check which index starts with _value
        if(!strncmp(values[i], _value, valueLength)) {
            return i;
        }
    }
    return -1;
}

int cRMSetupTools::Value2Index(eItemType _item,
        int _value, int _minValue, int _maxValue)
{
    int numIndices=NumIndices(_item);
    int minValue=_minValue;
    int maxValue=_maxValue;
    switch(_item) {
        case ePercentItem:
        case eSliderItem:
        case eBigSliderItem:
            if(_minValue==-1 && _maxValue==-1) {
                minValue=0;
                maxValue=100;
            }
            return (int)(1.0*(_value-minValue)*(numIndices-1)/(maxValue-minValue)+0.5);
        case eOffsetItem: {
            if(_minValue==-1 && _maxValue==-1) {
                minValue=-100;
                maxValue=100;
            }
            int indexCenter=(numIndices-1)/2;
            int center=(maxValue+minValue+1)/2;
            int maxOffset=(maxValue-minValue)/2;
            return (int)(1.0*(_value-center)*indexCenter/maxOffset+indexCenter+0.5);
        }
        case eBoolItem:
            if(_value==0) {
                return 0;
            }
            return 1;
        default:
            for(int i=numIndices-1; i>=0; i--) {
                int idxValue=Index2Value(_item, i);
                if(idxValue<=_value) {
                    return i;
                }
            } 
            return 0;
    }
    return -1;
}

int cRMSetupTools::OffsetValue2Index(eItemType _item,
        int _value, int _center, int _maxOffset)
{
    switch(_item) {
        case eOffsetItem:
            return Value2Index(_item, _value,
                    _center-_maxOffset, _center+_maxOffset);
        default:
            return -1;
    }
    return -1;
}

int cRMSetupTools::VideoBitrateValue(int _index)
{
    char* value=Index2StringValue(eVideoBitrateItem, _index);
    char* pos=strchr(value, '.');
    if(pos) {
        pos[0]=pos[1];
        pos[1]=0;    
    } else {
        free(value);
        return -1;
    }
    int retVal=atoi(value)*100000;
    free(value);
    return retVal;
}

int cRMSetupTools::AudioBitrateValue(int _index)
{
    char* value=Index2StringValue(eAudioBitrateItem, _index);
    char* pos=strchr(value, 'k');
    if(pos) {
        pos[0]=0;    
    } else {
        free(value);
        return -1;
    }
    int retVal=atoi(value)*1000;
    free(value);
    return retVal;
}

